﻿#Region ;************ Includes ************
#include-once
#include <Date.au3>
#include <FileConstants.au3>
#include <GDIPlus.au3>
#include <GDIPlusConstants.au3>
#include <WinAPIGdiDC.au3>
#include <WinAPIGdiInternals.au3>
#include <WinAPIHObj.au3>
#include <WinAPISysWin.au3>
#include <WindowsConstants.au3>
#EndRegion ;************ Includes ************

If $__g_hGDIPDll = 0 Then
	_GDIPlus_Startup()
	OnAutoItExitRegister('__GUICtrlGifAnim_Exit')
EndIf
Global Const $__g_sGUICtrlGifAnim_Version = '1.0.0.0'
Global Const $__g_sGUICtrlGifAnim_Date = '2022/08/31 17:00:00'

Func __GUICtrlGifAnim_Exit()
	_GDIPlus_Shutdown()
EndFunc   ;==>__GUICtrlGifAnim_Exit

; #FUNCTION# ====================================================================================================================
; Name ..........: _GUICtrlGifAnim_Create
; Syntax ........: _GUICtrlGifAnim_Create($vGifAnim, $iX, $iY, $hParent)
; Description ...: Diese Funktion liest die Gif-Animation ein und erstellt aus allen Frames einen Backbuffer (HBitmap),
;                  der dann mit _GUICtrlGifAnim_DrawNextFrame stueckchenweise in das Layered-Window geblittet wird.
; Parameter .....: $vGifAnim = Eine Variable mit dem Pfad oder mit den Binaerdaten zur Gif-Animation.
;                  $iX, $iY = Positionen: Left und Top, an denen das Window erstellt werden soll.
;                  $hParent = Das Parent-Window.
; Return Value(s): Bei Erfolg = Eine Struktur, die fuer den Aufruf der anderen Funktionen benoetigt wird.
;                  Bei Fehlern = False und @error > 0
;                                @error = 1 $hParent ist kein Window-Handle
;                                @error = 2 Die Datei $vGifAnim kann nicht geoeffnet werden
;                                @error = 3 Die GDI+ Bitmap (Gif-Animation) konnte nicht erstellt werden
;                                @error = 4 Das Array mit den Framedelay-Werten konnte nicht erstellt werden
; Author(s) .....: Oscar (www.autoit.de)
; ===============================================================================================================================
Func _GUICtrlGifAnim_Create($vGifAnim, $iX, $iY, $hParent)
	Local Const $GDIP_PROPERTYTAGFRAMEDELAY = 0x5100, $iTransColor = 0x00FF00
	Local $hFile, $hImage, $iW, $iH, $aDelay
	If Not IsHWnd($hParent) Then Return SetError(1, 0, False)
	If FileExists($vGifAnim) Then
		$hFile = FileOpen($vGifAnim, $FO_BINARY)
		If $hFile = -1 Then Return SetError(2, 0, False)
		$vGifAnim = FileRead($hFile)
		FileClose($hFile)
	EndIf
	$hImage = _GDIPlus_BitmapCreateFromMemory($vGifAnim)
	If @error Then Return SetError(3, 0, False)
	$iW = _GDIPlus_ImageGetWidth($hImage)
	$iH = _GDIPlus_ImageGetHeight($hImage)
	$aDelay = _GDIPlus_ImageGetPropertyItem($hImage, $GDIP_PROPERTYTAGFRAMEDELAY)
	If @error Then
		_GDIPlus_ImageDispose($hImage)
		Return SetError(4, 0, False)
	EndIf
	Local $tAnim = DllStructCreate('hwnd hWnd; handle hDC; handle hHBitmap; ' & _
			'handle hDCBB; handle oDC; handle hGfx; uint width; uint height; ' & _
			'uint Count; uint Frame; uint Delay[' & $aDelay[0] & ']; dword LastMillis')
	$tAnim.hWnd = GUICreate('', $iW, $iH, $iX, $iY, $WS_POPUP, BitOR($WS_EX_MDICHILD, $WS_EX_LAYERED), $hParent)
	GUISetBkColor($iTransColor, $tAnim.hWnd)
	_WinAPI_SetLayeredWindowAttributes($tAnim.hWnd, $iTransColor)
	$tAnim.hDC = _WinAPI_GetDC($tAnim.hWnd)
	$tAnim.hHBitmap = _WinAPI_CreateCompatibleBitmap($tAnim.hDC, $aDelay[0] * $iW, $iH)
	$tAnim.hDCBB = _WinAPI_CreateCompatibleDC($tAnim.hDC)
	$tAnim.oDC = _WinAPI_SelectObject($tAnim.hDCBB, $tAnim.hHBitmap)
	$tAnim.hGfx = _GDIPlus_GraphicsCreateFromHDC($tAnim.hDCBB)
	$tAnim.width = $iW
	$tAnim.height = $iH
	_GDIPlus_GraphicsSetSmoothingMode($tAnim.hGfx, $GDIP_SMOOTHINGMODE_HIGHQUALITY)
	_GDIPlus_GraphicsSetPixelOffsetMode($tAnim.hGfx, $GDIP_PIXELOFFSETMODE_HIGHQUALITY)
	_GDIPlus_GraphicsClear($tAnim.hGfx, BitOR(0xFF000000, $iTransColor))
	$tAnim.Count = $aDelay[0]
	For $i = 0 To $aDelay[0] - 1
		$tAnim.Delay($i + 1) = $aDelay[$i + 1] * 10
		_GDIPlus_ImageSelectActiveFrame($hImage, $GDIP_FRAMEDIMENSION_TIME, $i)
		_GDIPlus_GraphicsDrawImage($tAnim.hGfx, $hImage, $i * $iW, 0)
	Next
	_GDIPlus_ImageDispose($hImage)
	$tAnim.Frame = 1
	$tAnim.LastMillis = _Date_Time_GetTickCount()
	Return $tAnim
EndFunc   ;==>_GUICtrlGifAnim_Create

; #FUNCTION# ====================================================================================================================
; Name ..........: _GUICtrlGifAnim_Delete
; Syntax ........: _GUICtrlGifAnim_Delete($tAnim)
; Description ...: Diese Funktion gibt die benutzten Ressourcen wieder frei.
; Parameter .....: $tAnim = die Struktur von _GUICtrlGifAnim_Create
; Author(s) .....: Oscar (www.autoit.de)
; ===============================================================================================================================
Func _GUICtrlGifAnim_Delete(ByRef $tAnim)
	_WinAPI_SelectObject($tAnim.hDCBB, $tAnim.oDC)
	_GDIPlus_GraphicsDispose($tAnim.hGfx)
	_WinAPI_DeleteObject($tAnim.hHBitmap)
	_WinAPI_ReleaseDC($tAnim.hWnd, $tAnim.hDC)
	GUIDelete($tAnim.hWnd)
EndFunc   ;==>_GUICtrlGifAnim_Delete

; #FUNCTION# ====================================================================================================================
; Name ..........: _GUICtrlGifAnim_SetState
; Syntax ........: _GUICtrlGifAnim_SetState($tAnim, $iState)
; Description ...: Diese Funktion setzt den Status fuer das Fenster der Gif-Animation
; Parameter .....: $tAnim = die Struktur von _GUICtrlGifAnim_Create
;                  $iState = entspricht den Flags bei GUISetState
; Author(s) .....: Oscar (www.autoit.de)
; ===============================================================================================================================
Func _GUICtrlGifAnim_SetState(ByRef $tAnim, $iState)
	GUISetState($iState, $tAnim.hWnd)
EndFunc

; #FUNCTION# ====================================================================================================================
; Name ..........: _GUICtrlGifAnim_DrawNextFrame
; Syntax ........: _GUICtrlGifAnim_DrawNextFrame($tAnim)
; Description ...: Diese Funktion prueft, ob die Anzeigedauer fuer einen Frame bereits erreicht ist. Wenn ja, wird der
;                  naechste Frame angezeigt.
; Parameter .....: $tAnim = die Struktur von _GUICtrlGifAnim_Create
; Author(s) .....: Oscar (www.autoit.de)
; ===============================================================================================================================
Func _GUICtrlGifAnim_DrawNextFrame(ByRef $tAnim)
	Local $iMillis = _Date_Time_GetTickCount()
	If $iMillis - $tAnim.LastMillis < $tAnim.Delay($tAnim.Frame) Then Return
	$tAnim.LastMillis = $iMillis
	$tAnim.Frame += 1
	If $tAnim.Frame > $tAnim.Count Then $tAnim.Frame = 1
	_WinAPI_BitBlt($tAnim.hDC, 0, 0, $tAnim.width, $tAnim.height, $tAnim.hDCBB, ($tAnim.Frame - 1) * $tAnim.width, 0, $SRCCOPY)
EndFunc   ;==>_GUICtrlGifAnim_DrawNextFrame
